@@ -148,9 +148,10 @@ def _pgcli(config, conn, params, use_docker_container=None):
 @db.command(name='reset-odoo-db')
 @click.argument('dbname', required=False)
 @click.option('--do-not-install-base', is_flag=True)
+@click.option('-C', "--collateC", is_flag=True, help="Use Collate C like odoo suggests")
 @pass_config
 @click.pass_context
-def reset_db(ctx, config, dbname, do_not_install_base):
+def reset_db(ctx, config, dbname, do_not_install_base, collatec):
     dbname = dbname or config.dbname
     if not dbname:
         raise Exception("dbname required")
@@ -160,9 +161,12 @@ def reset_db(ctx, config, dbname, do_not_install_base):
     conn = config.get_odoo_conn().clone(dbname=dbname)
     _dropdb(config, conn)
     conn = config.get_odoo_conn().clone(dbname='postgres')
+    cmd = ""
+    if collatec:
+        cmd2 = "LC_CTYPE 'C' LC_COLLATE 'C' ENCODING 'utf8' TEMPLATE template0"
+    cmd = f"create database {dbname} {cmd2} "
     _execute_sql(
-        conn,
-        f"create database {dbname}",
+        conn, cmd,
         notransaction=True
     )
 
