@@ -1,3 +1,11 @@
+"""
+To work the volumes folder must be a child of a zfs filesystem:
+zfs create zfs_pool1/docker
+zfs create zfs_pool1/docker/volumes
+
+set /etc/odoo/settings ZFS_PATH_VOLUMES=zfs_pool1/docker/volumes  then
+
+"""
 from operator import itemgetter
 import subprocess
 import arrow
@@ -47,56 +55,33 @@ def _get_path(config):
     path = DOCKER_VOLUMES / __get_postgres_volume_name(config)
     return path
 
+def _get_zfs_path(config):
+    path = config.ZFS_PATH_VOLUMES + "/" + __get_postgres_volume_name(config)
+    return path
 
 def _get_next_snapshotpath(config):
     counter = 0
     while True:
-        path = _get_path(config)
-        path = Path(str(path) + f".{counter}")
-        if not path.exists():
+        path = _get_zfs_path(config)
+        path = str(path) + f".{counter}"
+        if not __is_zfs_fs(path):
             break
         counter += 1
     return path
 
 
-def _get_possible_snapshot_paths(path):
+def _get_possible_snapshot_paths(config):
     """
     :param path: root path
     """
-    yield Path(path)
-    yield from Path(path).parent.glob(f"{path.name}.*")
-
-
-def _get_poolname_of_path(path):
-    """
-    output of "df -T <path>"
-    rpool/ROOT/ubuntu_srb0yj/var/lib/marc1 zfs  1689338880   128 1689338752   1% /var/lib/marc1
-
-    output of df -T /var/lib/marc1/abcdef
-    rpool/ROOT/ubuntu_srb0yj/var/lib/marc1 zfs  1689338880   128 1689338752   1% /var/lib/marc1
-
-    Poolname: rpool/ROOT/ubuntu_srb0yj
-    """
-    while str(path).endswith("/"):
-        path = Path(str(path[:-1]))
-    if Path(path).exists():
-        df = search_env_path("df")
-        dfout = subprocess.check_output([df, "-T", path], encoding="utf8").splitlines()[1]
-    else:
-        return None
-    fullpath, fstype, _, _, _, _, df_path = unify(dfout.strip()).split(" ")
-    poolname = str(fullpath).replace(df_path, "")
-    assert fullpath.startswith(poolname)
-
-    if df_path != str(path):
-        raise NotZFS(
-            f"path is not a zfs volume {path}.\n\n{HOWTO_PREPARE}", poolname=poolname
-        )
-    if fstype != "zfs":
-        raise Exception("Not a zfs filesystem")
-
-    return poolname
-
+    all_zfs_folders = _get_all_zfs()
+    zfs_path = _get_zfs_path(config)
+    root_zfs_path = '/'.join(zfs_path.split("/")[:-1])
+    for folder in all_zfs_folders:
+        if folder == zfs_path:
+            yield folder
+        if folder.startswith(zfs_path + "."):
+            yield folder
 
 def __get_postgres_volume_name(config):
     return f"{config.project_name}_odoo_postgres_volume"
@@ -105,17 +90,16 @@ def __get_postgres_volume_name(config):
 def __get_snapshots(config):
     path = _get_path(config)
     try:
-        return _get_snapshots(path)
+        return _get_snapshots(config)
     except NotZFS:
         abort(f"Path {path} is not a zfs.")
 
 
-def _get_snapshots(path):
+def _get_snapshots(config):
     zfs = search_env_path("zfs")
-    poolname = _get_poolname_of_path(path)
 
-    def _get_snaps(path):
-        for path in _get_possible_snapshot_paths(path):
+    def _get_snaps():
+        for path in _get_possible_snapshot_paths(config):
             for line in (
                 subprocess.check_output(
                     ["sudo", zfs, "list", "-t", "snapshot", str(path)],
@@ -140,66 +124,66 @@ def _get_snapshots(path):
                 info["date"] = timestamp
                 info["fullpath"] = snapshotname
                 info["name"] = snapshotname.split("@")[1]
-                info["pool"] = poolname
-                info["path"] = snapshotname.replace(poolname, "").split("@")[0]
+                info["path"] = snapshotname.split("/")[-1]
                 yield info
-    yield from sorted(_get_snaps(path), key=lambda x: x['date'], reverse=True)
+    yield from sorted(_get_snaps(), key=lambda x: x['date'], reverse=True)
 
 
+def _get_all_zfs():
+    zfs = search_env_path("zfs")
+    folders = subprocess.check_output(["sudo", zfs, "list"], encoding="utf8").splitlines()
+    folders = list(map(lambda x: x.split(" ")[0], folders))
+    return folders
+
+def __is_zfs_fs(path_zfs):
+    assert " " not in path_zfs
+    folders = _get_all_zfs()
+    folders = [x for x in folders if x.split(" ")[0] == path_zfs]
+    return folders
+
 def assert_environment(config):
     pass
 
 
-def _turn_into_subvolume(path):
+def _turn_into_subvolume(config):
     """
     Makes a zfs pool out of a path.
     """
-    if not path.exists():
-        abort("Please start postgres container at least once")
-    try:
-        _get_poolname_of_path(path)
-    except NotZFS as ex:
-        zfs = search_env_path("zfs")
-        click.secho(f"Turning {path} into a zfs pool.", fg="green")
-        filename = path.parent / Path(tempfile.mktemp()).name
-        if filename.exists():
-            raise Exception(f"Path {filename} should not exist.")
-        shutil.move(path, filename)
-        try:
-            assert str(path).startswith("/")
-            subprocess.check_call(["sudo", "mkdir", path])
-            fullpath = ex.poolname + str(path)
-            try:
-                subprocess.check_output(["sudo", zfs, "create", fullpath])
-            except Exception as ex:
-                if 'dataset already exists' in str(ex):
-                    subprocess.check_output(["sudo", zfs, "destroy", fullpath])
-                subprocess.check_output(["sudo", zfs, "create", fullpath])
-
-            click.secho(
-                f"Writing back the files to original position: from {filename}/ to {path}/"
-            )
-            subprocess.check_call(
-                [
-                    "sudo",
-                    "rsync",
-                    str(filename) + "/",
-                    str(path) + "/",
-                    "-ar",
-                ]
-            )
-        finally:
-            subprocess.check_call(["sudo", "rm", "-Rf", filename])
-    else:
+    zfs = search_env_path("zfs")
+    fullpath = _get_path(config)
+    fullpath_zfs = _get_zfs_path(config)
+    if __is_zfs_fs(fullpath_zfs):
+        # is zfs - do nothing
         return
 
+    filename = fullpath.parent / Path(tempfile.mktemp()).name
+    if filename.exists():
+        raise Exception(f"Path {filename} should not exist.")
+    shutil.move(fullpath, filename)
+    try:
+        subprocess.check_output(["sudo", zfs, "create", fullpath_zfs])
+        click.secho(
+            f"Writing back the files to original position: from {filename}/ to {fullpath}/"
+        )
+        subprocess.check_call(
+            [
+                "sudo",
+                "rsync",
+                str(filename) + "/",
+                str(fullpath) + "/",
+                "-ar",
+            ]
+        )
+    finally:
+        subprocess.check_call(["sudo", "rm", "-Rf", filename])
+
 
 def make_snapshot(ctx, config, name):
     zfs = search_env_path("zfs")
     __dc(["stop", "-t 1"] + ["postgres"])
     path = _get_path(config)
-    _turn_into_subvolume(path)
-    snapshots = list(_get_snapshots(path))
+    _turn_into_subvolume(config)
+    snapshots = list(_get_snapshots(config))
     snapshot = list(filter(lambda x: x["name"] == name, snapshots))
     if snapshot:
         if config.force:
@@ -208,9 +192,8 @@ def make_snapshot(ctx, config, name):
             click.secho(f"Snapshot {name} already exists.", fg="red")
             sys.exit(-1)
 
-    poolname = _get_poolname_of_path(path)
     assert " " not in name
-    fullpath = f"{poolname}{path}@{name}"
+    fullpath = _get_zfs_path(config) + "@" + name
     subprocess.check_call(["sudo", zfs, "snapshot", fullpath])
     __dc(["up", "-d"] + ["postgres"])
     return name
@@ -218,20 +201,20 @@ def make_snapshot(ctx, config, name):
 
 def restore(config, name):
     zfs = search_env_path("zfs")
+    umount = search_env_path("umount")
     if not name:
         return
 
     assert "@" not in name
     assert "/" not in name
 
-    path = _get_path(config)
-    snapshots = list(_get_snapshots(path))
-    default_volume_path = Path(_get_path(config))
+    snapshots = list(_get_snapshots(config))
     snapshot = list(filter(lambda x: x["name"] == name, snapshots))
     if not snapshot:
         abort(f"Snapshot {name} does not exist.")
     snapshot = snapshot[0]
-    snapshots_of_volume = [x for x in snapshots if Path(x['path']) == default_volume_path]
+    zfs_full_path = _get_zfs_path(config)
+    snapshots_of_volume = [x for x in snapshots if x['fullpath'].split("@")[0].startswith(zfs_full_path)]
     try:
         index = list(map(lambda x: x['name'], snapshots_of_volume)).index(name)
     except ValueError:
@@ -242,14 +225,15 @@ def restore(config, name):
         # restore last one is easy in the volumefolder it self; happiest case
         subprocess.check_call(["sudo", zfs, "rollback", snapshot["fullpath"]])
     else:
-        next_path = _get_next_snapshotpath(config)
-        poolname = _get_poolname_of_path(snapshot["path"])
-        full_next_path = poolname + str(next_path)
+        full_next_path = _get_next_snapshotpath(config)
         subprocess.check_call(
-            ["sudo", zfs, "rename", poolname + str(path), full_next_path]
+            ["sudo", umount, zfs_full_path],
         )
         subprocess.check_call(
-            ["sudo", zfs, "clone", snapshot['fullpath'], poolname + str(path)]
+            ["sudo", zfs, "rename", zfs_full_path, full_next_path]
+        )
+        subprocess.check_call(
+            ["sudo", zfs, "clone", full_next_path + "@" + snapshot['name'], zfs_full_path]
         )
     __dc(["rm", "-f"] + ["postgres"])
     __dc(["up", "-d"] + ["postgres"])
@@ -270,12 +254,11 @@ def remove(config, snapshot):
 
 def remove_volume(config):
     zfs = search_env_path("zfs")
-    volume_path = _get_path(config)
-    pool_name = _get_poolname_of_path(volume_path)
-    if not pool_name:
-        return
-    for path in _get_possible_snapshot_paths(volume_path):
-        if not path.exists():
-            continue
-        subprocess.check_call(["sudo", zfs, "destroy", "-R", pool_name + str(path)])
+    umount = search_env_path("umount")
+    for path in _get_possible_snapshot_paths(config):
+        try:
+            subprocess.check_call(["sudo", umount, path])
+        except:
+            pass
+        subprocess.check_call(["sudo", zfs, "destroy", "-R", path])
         click.secho(f"Removed: {path}", fg='yellow')
